C--------------------------------------------------------------------
C     SOLVE EQUATION (10) IN REFERENCE.
C calculate derivatives dC/drho, and with them the new density
C new density of given pixel = old den * EXP(-lambda*(derivative))
C with derivate = sum dC/drho of all constraints: F- and G-type
C--------------------------------------------------------------------
C 
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints (F=A+iB)
C*   SIGF(1..nrefF) contains the sigmas(|F|) of the F-constraints
C*   GOBS(1..nrefG) contains the mean |F| of G-constraints
C*     SIGG(1..nrefG) contains the sigma of Gobs of group j
C*     NGRP(1..nrefG) contains the number of reflections in group j
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j
C*   nrefF,nrefG contain the numbers of constraints: F, G
C*   nref=total number of reflections>=nrefF+nrefG
C*   FCAL(1..2,1..nref) contains the calculated A and B of ALL constraints
C*   GCAL(1..nrefG) contains the calculated |G| of the G-constraints
C
C*   RDEOp,RDEOm are the old density values
C*   RDENp,RDENm are the new density values
C
      SUBROUTINE MAXENT(ngsw,mref,nrefF,nrefG,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,tnref,lambda
     $  ,RMIC
CENT $  ,RMIS
     $  ,PMUL,PMULS,RDEOp,RDEOm,RDENp,RDENm,
     $   RHELP1,RHELP2,RHELP3,denmin,sumBp,sumBm,rtBp,rtBm)
C
      IMPLICIT none
      integer ngsw,mref,jref,iref,nrefF,nrefG,igrp
      integer*4 masym,nasym,ia
C     *** density arrays ***
      real*8 RDENp(masym),RDENm(masym),RDEOp(masym),RDEOm(masym)
     $   ,rtasym,rtBp,rtBm
C     *** structure factor lists ***
      real*8 FOBS(2,mref),FCAL(2,mref),SIGF(mref),PROF(mref),
     $         GOBS(mref),GCAL(mref),SIGG(mref)
      integer NGRP(mref)
      real*8 PMUL(mref),PMULS(mref)
C     *** total number of Fobs/Gobs given 
      real*8 tnref    
C     *** scattering length sums, positive and negative:
      real*8 sumBp, sumBm
C     *** cosine and sinus-tables ***
      real*4 RMIC(mref,masym)
CENT $      ,RMIS(mref,masym)
C     *** auxilary arrays ***
      real*8 RHELP1(mref),RHELP2(mref),RHELP3(mref) 
      real*8 sum1,lambda,derf,derg,denmin,d
C
C     *** precalculations ***
C
C
C     *** for F-constraints ***
      do 1 iref=1,nrefF
        RHELP1(iref)= (FCAL(1,iref)*PROF(iref)-FOBS(1,iref))
     $	          /(SIGF(iref)*rtasym)
CENT    RHELP2(iref)= (FCAL(2,iref)*PROF(iref)-FOBS(2,iref))
CENT $            /(SIGF(iref)*rtasym)
1     continue
C
C     *** for G-constraints ***
      IF (ngsw.EQ.0) THEN
        jref=nrefF+1
        DO 3 iref=1,nrefG
          DO 2 igrp=1,NGRP(iref)
            RHELP1(jref)=PMUL(jref-nrefF)*FCAL(1,jref)*PROF(jref)
CENT        RHELP2(jref)=PMUL(jref-nrefF)*FCAL(2,jref)*PROF(jref)
            jref=jref+1
2         continue
          RHELP3(iref)= (GCAL(iref)-GOBS(iref))/
     $          (GCAL(iref)*SIGG(iref)*PMULS(iref)*rtasym)
3       continue
      ENDIF
C     *** end precalculations ***
C
C     *** now calculate derivatives for each point in asymmetric unit ***
C
      derf=0.0
      derg=0.0
      DO 10 ia=1,nasym
C        *** DERIVATIVE FOR F-TYPE CONSTRAINT ***
         derf=0.0d0
         DO 20 iref=1,nrefF
           derF=derF + RHELP1(iref)*RMIC(iref,ia)
CENT $               + RHELP2(iref)*RMIS(iref,ia)
   20    CONTINUE
         if (nrefF.gt.0) derF=derF/nrefF
C
C        *** DERIVATIVE FOR G-TYPE CONSTRAINT ***
         IF (ngsw.EQ.0) THEN
         derG=0.0d0
         jref=nrefF+1
         DO 40 iref=1,nrefG
           sum1=0.0
           DO 50 igrp=1,NGRP(iref)
             sum1=sum1 + RHELP1(jref)*RMIC(jref,ia)
CENT $                 + RHELP2(jref)*RMIS(jref,ia)
             jref=jref+1
   50      CONTINUE
           derG=derG+sum1*RHELP3(iref)
   40    CONTINUE
         if (nrefG.gt.0) derG=derG/nrefG
         END IF
C
C*** SOLVE EQ.(10) IN REFERENCE. ***
C
         RDENp(ia)=RDEOp(ia)*DEXP(-lambda*sumBp*(derF+derG))
         RDENm(ia)=RDEOm(ia)*DEXP(-lambda*sumBm*(derF+derG))
C
C        *** check, whether both the positive and the negative part
C        *** of the density part are large, and put the density where
C        *** it belongs: if the sum is negative, attach it to the negative
C        *** part and reset the positive density to its minimum value.

         d=rtBp*RDENp(ia) + rtBm*RDENm(ia)	!real density sum
         
C        *** does not work:!!! ***
         if (1.eq.2) then
         if (d.lt.0) then 
           if (rtBp.gt.0) RDENp(ia)=denmin/rtBp	!reset positive density
C          if (rtBm.lt.0) RDENm(ia)=d/rtBm	!d and rtBm are negative
         else
           if (d.eq.0) then
              RDENp(ia)=denmin/rtBp		!reset both densities
              RDENm(ia)=denmin/abs(rtBm)
           else
C             *** d>0 
C             if (rtBp.gt.0) RDENp(ia)=d/rtBp		!d and rtBp are positive
              if (rtBm.lt.0) RDENm(ia)=denmin/abs(rtBm)	!reset negative density
           endif
         endif
         endif
C
C        *** check underflow ***
         if (rtBp.gt.0) then
           if (RDENp(ia)*rtBp.lt.denmin) RDENp(ia)=denmin/rtBp 
         endif
         if (rtBm.lt.0) then
           if (RDENm(ia)*abs(rtBm).lt.denmin) 
     $         RDENm(ia)=denmin/abs(rtBm) 
         endif
C
   10 CONTINUE
C
      RETURN
      END
