C========================================================================
C
C     mendc.f -- for non-centrosymmetric samples
C                NEUTRON densities version
C
C     MAXIMUM ENTROPY ELECTRON DENSITY ANALYSIS PROGRAM
C     for neutron powder or single crystal diffraction data
C     REFERENCE : J.APPL.CRYST. (1990) 23, 526-534
C               : J.APPL.CRYST. (1993) 26, 453-457 (MEED program)
C               : J.Appl.Cryst. (1993) 26, 159-165 (neutron version)
C               : K. Burger, Dissertation/thesis, Universitaet Tuebingen, 1997
C                 (published by Shaker, Aachen, Germany).
C     COPYRIGHT BY SHINTARO KUMAZAWA ( NAGOYA UNIVERCITY )
C     ALL RIGHT RESERVED.
C     modified and enhanced by K. Burger, Inst. fuer Kristallographie,
C     Universitaet Tuebingen, Charlottenstr. 33, D-72070 Tuebingen,
C     Germany. 
C     Free use for academic users.
C     See the Tuebingen web-pages: http://www.uni-tuebingen.de/uni/pki
C
C     - UPDATE LABEL -
C
C     VERSION 1.00  1990-04-24  BY SHINTARO KUMAZAWA
C      ORIGINAL PROGRAM, MEMTARO.FORT.
C     VERSION 1.10  1990-11-08  BY YOSHIKI KUBOTA
C      ADD PREFERED ORIENTATION CORRECTION VERSION.
C     VERSION 2.00  1991-10-16  BY SHINTARO KUMAZAWA
C      START MEED SYSTEM PROJECT.
C      CHANGE SYMMETRY TREATMENT AND SOME CALCULATION ALGORITHM.
C     VERSION 2.12  1991-10-25  BY SHINTARO.KUMAZAWA
C      BUG FIX & OPTIMIZE SOME CALCULATION ALGORITHM.
C     VERSION 3.00  1991-11-15  BY SHINTARO KUMAZAWA
C      MODIFIED BY USING OF MAU METHOD.
C     Data input now read free-format. Better handling of divergence errors 
C      - 12.1995 k.b.
C     Data output format changed  - K. Burger 3.1996
C     MaxEnt-routine optimized for velocity.
C      New COMMON variables: rhelp1,2,4,5,6(mref) for MAXENT-SUBROUTINE.
C      Changes within loop over maxent cycles.
C      Memorize FCAL and GCAL from last successful cycle in case of
C      divergence - K. Burger 3.96 
C     lambda is varied automatically: in case of divergence lambda=lambda*0.5
C      if cycle was successful: lambda is increased a bit.
C      Bug in MKASYM routine fixed: wrong results in the case of
C      different pixel resolution: NPIX(1)<>NPIX(2 or 3).
C      Translation vectors ITRS and ISUB now real numbers: RTRS and RSUB.
C      Program split into modules; no COMMON blocks any more. (k.b. 4.1996)
C     Built in test: if constraint changes too quickly (by more than 10 %),
C      then the cycle is recalculated.  - Dec. 1996, K. Burger
C     Fixed bug with constraint-value calculation with GOBS in first cyles.
C      Built in deviation Fcal-Fobs in sigma units in display of results. 
C      K.B. Aug./Sept. 1997
C     NEUTRON version March 1998 (K. Burger)
C     RMIC and RMIS changed to single-precision variables (4-byte reals)
C      which saves half the computer memory (k.b. 4-98)
C
C     - CAUTION !! -
C
C     (1) IF YOU TRY A SAMPLE HAVING NO CENTRE OF SYMMETRY, YOU MUST
C         REMOVE THE COMMENT MARK, 'CENT'.
C         Also, calls to subroutines MKASYM, STRCTF,STRCTG,CALCON,MAXENT
C         have to be substituted to their corresponding ones CMKASYM,CSTRCF,
C         CSTRCG,CCALCON,CMAXENT
C           
C     (2) YOU MAY NOT USE AN OLD FORMAT MEM DATA SET.
C
C     (3) SUBROUTINES DATE & WATCH CAN BE ONLY USED BY FACOM COMPUTER.
C
C     - UNIT NUMBER OF DATA FILES -
C
C     10 : MEND DATA SET ( THIS IS POWLS6.EXE COMPATIBLE FORMAT. )
C     20 : DENSITY DATA SET ( THIS IS READABLE BY MECONT.EXE. )
C     30 : ANALYSIS DOCUMENT DATA SET
C     40 : SPACE GROUP DATA BASE 'SPGRA.DAT', OFFERED BY FUJIO IZUMI.
C          (needed in subroutine rdspg.f)
C
C====================================================================
C
C
      PROGRAM mend
C
C       PARAMETERS:
C       mref = max. number of reflections
C       masym = max. number of pixels in asymmetric unit of cell
C       mpix = max. number of pixels along one cell axis (resolution)
C       mhist = max. number of changes in lambda-parameter done by the user
C       alpha = Final value of constraint
C       denmin = minimal value of normalized density
C       mautosave = number of cycles, after which an automatic savig of the 
C               current density is done.
C
      IMPLICIT none
C
      integer*4 nasym,masym,ia
      integer mhist,mref,mpix,mautosave
      real*8 alpha,denmin
C
      PARAMETER ( mref= 50, masym=60000, mpix=65)
      PARAMETER ( mhist=50, mautosave=100 )
      PARAMETER ( alpha=1.0, denmin=1.0e-5 )
C
C     *** VARIABLES ***
C
C* - The list of F-constraints: h k l A B sigma(|F|) (phased reflections)
C* - The list of G-constraints: hkl-list + 1 Gobs and s(Gobs) (groups of refl.)
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints
C*   sigF(1..nrefF) contains the sigma(|F|)
C*   GOBS(mref) contains the mean |F| of G-constraints: GOBS(1..nrefG)
C*     NGRP(1..nrefG) contains the number of reflections in group j
C*     SIGG(1..nrefG) contains the sigma of Gobs of group j
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j
C*   nrefF,nrefG contain the numbers of constraints: F, G
C*   nref=total number of reflections
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell:
      real*8 RDEOp(masym),RDEOm(masym),     ! starting density (positive+neg.)
     +       RDENp(masym),RDENm(maSYM),     ! new density 
     +       RASYM(masym) 
C
C     *** variables connected with the coordinates of the pixels:
      integer NPIX(3),IASYM(3,masym)
C
C     *** variables connected with the structure factors given as input:
C     *** FOBS: unique reflections, GOBS: groups of non-unique refl.
      real*8 FOBS(2,mref),FCAL(2,mref),GCAL(mref),
     $    SIGF(mref),PROF(mref),GOBS(mref),SIGG(mref)
      real*8 tnref
      integer IDX(3,mref),NGRP(mref),nref,nrefF,nrefG
C
C     *** cell-constants and others:
      real*8 CELL(6),pop,rtasym
      integer jph,jpk,jpl
      real*8 pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,sumBp,sumBm,rtBp,rtBm
C
C     *** variables connected with the entropy-functions:
      real*8 rfact,wrfact,entsp,entsm,entj,qfunc,conf,cong,connew,
     $  delta,lambda
C
C     *** arrays to memorize lquantities of last successful cycle 
      real*8 FCALO(2,mref),GCALO(mref),confold,deltaold,rfacold,
     $  wrfacold,conold,congold
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** tables of cos- and sin-values for each reflection and pixel:
C     *** changed to contain only single-precision values (k.b. 4/98)
      real*4 RMIC(mref,masym)
      real*4 RMIS(mref,masym)
C
C     *** variables connected with lambda-history list:
      integer IHIST(MHIST),nhist
      real*8 FHIST(3,MHIST)
C
C     *** variables connected with multiplicities of reflections given as groups
C     of overlayd reflections (G_obs, G-constraints):
      real*8 PMUL(mref),PMULS(mref)
C
C     *** auxilary arrays for maxent subroutine
      real*8 RHELP1(mref),RHELP2(mref),RHELP3(mref)
C
C     *** auxilary array for mkasym subroutine
      integer IDE(0:mpix,0:mpix,0:mpix)
C
C     *** other variables:
      CHARACTER title*200,filmem*50,filden*50,fildoc*50,
     $          aspg*10
      real rnum,rnumc,rnumi		!for memory requirement calc.
      character*10 APRIOR(2)
      character*5 AGSW(2)
      integer iprior,igsw,nyes,iptyp,nsg,nset,
     $  ierr,itcycl,ngsw,iref,nn,i,j,
     $  ncycl,iii,icycl,ibadruns,igoodcyc
C     integer ITODAY(4)
      DATA (APRIOR(J),J=1,2)/'FLAT.     ','FILE DATA. '/
      DATA (AGSW(J),J=1,2)/'F & G','  F  '/
C
C
 1090 format(2(3(I3,1X),2(1x,G15.8)))  ! for display of asym. unit pixels
 2010 FORMAT(BN,A50)                 ! for filenames
 2020 FORMAT(BN,I5)		     ! for input of yes/no=0/1
 3000 FORMAT(4(/' '))
 3010 FORMAT(/'   MMM            MMM   EEEEEEEEEEEEEEE   NNN       ',
     $   '  NNN   DDDDDDDDDD'/'   MMMMMM      MMMMMM   EEEEEEEEEE',
     $   'EEEEE   NNNN        NNN   DDDDDDDDDDDD'
     $  /'   MMM MMM    MMM MMM   EEE               NNNNN       NNN  ',
     $   ' DDD        DDD'
     $  /'   MMM  MMM  MMM  MMM   EEE               NNN N',
     $   'N      NNN   DDD         DDD'
     $  /'   MMM   MMMMMM   MMM   ',
     $   'EEEEEEEEEEEEEEE   NNN  NN     NNN   DDD          DDD'
     $  /'   MMM     MM     MMM   EEEEEEEEEEEEEEE   NNN   NN    ',
     $   'NNN   DDD          DDD'
     $  /'   MMM            MMM   EEE   ',
     $   '            NNN    NN   NNN   DDD         DDD'
     $  /'   MMM ',
     $   '           MMM   EEE               NNN     NN  NNN   DD',
     $   'D        DDD'
     $  /'   MMM            MMM   EEEEEEEEEEEEEEE ',
     $   '  NNN      NN NNN   DDDDDDDDDDDD'
     $  /'   MMM            MM',
     $   'M   EEEEEEEEEEEEEEE   NNN        NNNN   DDDDDDDDDD'
     $  /'   MaxEnt scattering density analysis program ',
     $  /  '   for neutron single crystal or powder diffraction data'
     $  // ' References:',
     $   /,' J.Appl.Cryst. (1990) 23, 526-534'
     $   /,' J.Appl.Cryst. (1993) 26, 159-165',
     $   /,' K. Burger, thesis, Universitaet Tuebingen 1997.',
     $  /' COPYRIGHT BY SHINTARO KUMAZ',
     $  'AWA (C) 1991, K. Burger 1998, ALL RIGHT RESERVED.',
     $  //' MENDC Version of K. Burger,'/' Institut fuer',
     $  ' Kristallographie, Universitaet Tuebingen',
     $  /' Charlottenstr. 33, D-72070 Tuebingen, Germany, ',
     $  ' (March a.d. 1998)'
     $  /,' >> for non-centrosymmetric crystals <<')
 3011 FORMAT(/' Current MEND program parameters:',
     $       /'  maximum number of reflections = mref = ',i6,
     $       /'  max. number of asym. unit pixels = masym = ',i7,
     $       /'  max. number of pixels along a cell axis = mpix = ',i4,
     $       /'  max. number of history list entries = mhist = ',i4,
     $       /'  autosave of density after each ',i5,' cycles',
     $       /'  target value of constraint function = alpha = ',f10.4
     $       /'  minimum density value = denmin = ',e14.6)
 3012 FORMAT(/' Current memory requirements for program variables:',
     $       /'  number of real variables in MEND    ',f15.0,
     $       /'  additional real variables in MENDC  ',f15.0,
     $       /'  number of integer variables  ',f15.0,
     $       /'  memory needed for MEND  ',f15.0,' Kbytes ',
     $       /'  memory needed for MENDC ',f15.0,' Kbytes ',
     $       /'  (assuming 8 byte reals and 4 byte integers, and ',
     $          '4-byte reals RMIC and RMIS)')
 3020 FORMAT(/' *********   INITIAL STATE OF ANALYSIS   ******',
     $       '**************************',
     $        /A80, /A80,/ A40/
     $        /' MEND DATA FILE NAME     = ',A50
     $        /' DENSITY DATA FILE NAME  = ',A50
     $        /' DOCUMENT DATA FILE NAME = ',A50
     $       //' CELL CONSTANT = ',5(F8.3,','),F8.3
     $        /' SPACE GROUP   = ',A10,
     $          ' Bsum+=',f12.3,' Bsum-=',f12.3,
     $        /' P.O. CORRECT  = ',F10.5,'  ,  TYPE = ',I2,
     $         '  ,  VECTOR = ',2(I3,','),I3
     $        /' NUMBER OF PIXELS = ',I7,' / ',I7,'   (',2(I3,','),
     $         I3,' )'
     $       //' REFLECTIONS OF CONSTRAINT FUNCTION  F-TYPE = ',I4,
     $         ' , G-TYPE = ',I4
     $        /' INITIAL DENSITY DISTRIBUTION IS ',A
     $        /' THIS SESSION IS ANALYZED BY USING ',A,' CONSTRAINT.'
     $        /' TRIAL LAGRANGE MULTIPLIER  lambda = ',F10.7,'.')
 3030 FORMAT( '  con=',F18.5,' conf=',F15.4,' cong=',F15.4
     $        /' delt=',F18.5,'    R=',F15.4,'   wR=',F15.4)
 3040 FORMAT(  ' ent+=',F18.7,' ent-=',f18.7,' ents=',f18.7,
     $        /' entj=',F18.7,' lagr=',F18.7)
 3060 FORMAT(' CYCLE ',I5)
 3130 FORMAT(/' *********   WAIT A MOMENT ! THE PHASE MATRIX ',
     $       'TABLE IS CALCULATED. ******')
 3500 FORMAT(/ ' INPUT MEND FILE NAME     = ',$)
 3510 FORMAT(  ' INPUT DENSITY FILE NAME  = ',$)
 3520 FORMAT(  ' INPUT DOCUMENT FILE NAME = ',$)
 3530 FORMAT(  ' SELECT INITIAL DENSITY DATA TYPE.  0=FLAT, 1=',
     $       'READ IN FILE DATA : ',$)
 3540 FORMAT(  ' DO YOU USE G-CONSTRAINTS',
     $       ' IN THE CALCULATION ? (0=YES, 1=NO) : ',$)
 3550 FORMAT(  ' ARE YOU SURE ?  (0=YES, 1=NO) : ',$)
 3560 FORMAT(/' INPUT NCYCL. ( >0:ITERATION CYCLES , <0:CHANGE LAMD',
     $       'A , =0:EXIT )  : ',$)
C
C
C*** INPUT FILENAMES AND INITIAL CONDITIONS. ***
C
      WRITE(6,3010)
  201 WRITE(6,3500)
      READ(5,2010) filmem
      WRITE(6,3510)
      READ(5,2010) filden
  203 WRITE(6,3520)
      READ(5,2010) fildoc
      OPEN(30,FILE=FILDOC,FORM='FORMATTED',STATUS='UNKNOWN',
     $     ERR=203,ACCESS='SEQUENTIAL')
      WRITE(30,3010)
      WRITE(6,3530)
      READ(5,2020) iprior
      WRITE(6,3540)
      READ(5,2020) igsw
      WRITE(6,3550)
      READ(5,2020) nyes
      IF (nyes.NE.0) GO TO 201
C
C*** READ DATA AND MAKE SOME CONSTANTS. ***
C
      ierr=0
206   CALL READMEM(filmem,title,CELL,pop,iptyp,jph,jpk,jpl,
     $  nsg,nset,NPIX,mpix,sumBp,sumBm,lambda,mref,nref,nrefF,nrefG,
     $  IDX,FOBS,SIGF,GOBS,SIGG,NGRP,PMUL,PMULS,ierr,igsw)
      if (ierr.ne.0) then 
        write(*,*) 'ERROR: MEM-file could not be opened: ',filmem
        write(*,*)'Please input a new name:'
        read(*,'(a)') filmem
        goto 206
      endif
C
      write(*,'(a,f12.4,a,f12.4)') 
     $ ' Scattering length sums: pos=',sumBp,'  neg=',sumBm
C
C     *** read symmetry info ***
      CALL RDSPG(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)
C
C     *** display of symmetry info ***
C     CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
C     *** precalculations / make some constants 
      CALL PRECAL(jph,jpk,jpl,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,sumBp,sumBm,rtBp,rtBm,CELL,NPIX)

      CALL POCOR(iptyp,pop,PROF,nref,mref,IDX,
     $	               as,bs,cs,ga,gb,gg,rh,rk,rl,pi)
C
C*** INITIALIZE DATA & OUTPUT INITIAL STATE. ***
C
  210 WRITE(6,3130)
      IF (iprior.EQ.0) THEN
         CALL CMKASYM(0,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDENp,RDENm,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      ELSE
	 CALL READDEN(filden,masym,nasym,IASYM,RDENp,RDENm,rtBp,rtBm)
         CALL CMKASYM(1,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDENp,RDENm,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      END IF
C     print *,' nsym,nsub,ncen'
C     print *,nsym,nsub,ncen,(npix(i),i=1,3)
C     write(*,'(12i5)') (((irot(i,j,k),I=1,3),j=1,3),
C    $  (nint(npix(i)*rtrs(i,k)),i=1,3),
C    $    k=1,nsym)
C     write(*,'(3i5)') ((nint(npix(i)*rsub(i,k)),i=1,3),k=1,nsub)
C     read(*,*) idummi
C
C     *****************************************
C     ***** do precalculations for maxent *****
C     *****************************************
C
      NHIST=0
      itcycl=1
      delta=-1.0D-9
C     *** set densities to zero if scattering sum is zero
      if (sumBp.le.0) then
         print *,'Positive density is set to zero '
         do ia=1,nasym
            RDENp(ia)=0.0
         end do
      endif
      if (sumBm.ge.0) then
         print *,'Negative density is set to zero '
         do ia=1,nasym
            RDENm(ia)=0.0
         end do
      endif
      if (sumBp.le.0 .and. sumBm.ge.0) 
     $  CALL PROEND(' Incorrect scattering length sums given!')
         
C     *** if only Fobs are given: ngsw=1
C     *** if Gobs are given: in the first cycle, ngsw=-1 and the Gobs are 
C     *** not considered in the maxent-process. This is because Gcal is 
C     *** zero for flat starting-density, and there would result a `divided
C     *** by zero' error in the MAXENT routine.
C     *** After the first cycle, ngsw is set 0 in this case.
      ngsw=1
      if (nrefG.gt.0) then
        if (iprior.eq.0) then
C         *** Gobs given and flat starting density in first cycle ***
          ngsw=-1
        else 
C         *** Gobs and non-flat starting density from file ***
          ngsw=0
        endif
      endif
C     *** rden --> rdeo; calculate and memorize F_calc ***
      CALL RENEW(masym,nasym,RDEOp,RDEOm,RDENp,RDENm)
      CALL CSTRCTF(masym,nasym,RDENp,RDENm
     $   ,RMIC
     $   ,RMIS
     $   ,RASYM,mref,nref,FCAL,sumBp,sumBm)
C     *** memorize FCAL
      DO 42 iref=1,nref
	 FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
42    CONTINUE
C
      NN=nrefF
      IF (NGSW.le.0) THEN
         NN=NN+nrefG
      ENDIF
      tnref=DFLOAT(NN)
      IF (NGSW.le.0) THEN
         CALL CSTRCTG( mref,nrefF,nrefG,FCAL,GCAL,
     $      PROF,NGRP,PMUL,PMULS)
C        *** memorize GCAL
         DO 43 iref=1,nrefG
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
43       CONTINUE
      ENDIF
C
C     *** calculate constraint values ***
      CALL CCALCON(ngsw,conf,cong,wrfact,rfact,connew,tnref,
     $   mref,nrefF,nrefG,FCAL,PROF,FOBS,SIGF,GCAL,GOBS,SIGG )
      conold=connew
C     *** calculate entropy-function values ***
      CALL CALFN(masym,nasym,entj,entsp,entsm,rtasym,
     $   RDENp,RDENm,RDEOp,RDEOm,RASYM,
     $   qfunc,lambda,connew)
C
C     *** output starting values ***
      DO 60 I=1,2
         NN=6
         IF (I.EQ.2) NN=30
         if (i.eq.2) then
C          *** show parameters ***
           write(nn,3011) mref,masym,mpix,mhist,mautosave,alpha   
C          it is assumed, that RMIC and RMIS are always real*4 variables
           rnum=20.*mref+3.*mhist+5.*masym+51+mref*masym*.5
           rnumc=rnum+mref*masym*.5	!add RMIS space for MEEDC version
           rnumi=(mpix+1.)**3+3.*masym+3*3*48+mhist+4.*mref+34
           write(nn,3012) rnum, rnumc-rnum, rnumi,
     $       (rnum*8.+rnumi*4.)/1024., (rnumc*8.+rnumi*4.)/1024. 
         endif
         WRITE(NN,3020)
     $   title(1:80),title(81:160),title(161:200),
     $   filmem,filden,fildoc,(CELL(j),j=1,6),aspg,sumBp,sumBm,
     $   pop,iptyp,jph,jpk,jpl,
     $   nasym,NPIX(1)*NPIX(2)*NPIX(3),(NPIX(j),j=1,3),
     $   nrefF,nrefG,APRIOR(iprior+1),AGSW(igsw+1),lambda
         WRITE(NN,3030) connew,conf,cong,delta,rfact,wrfact
         WRITE(NN,3040) entsp,entsm,entsp+entsm,entj,qfunc
   60 CONTINUE
C
C     ************************************************************
C     *** MAXENT PROCESS ***
C     ************************************************************
C
C     ***** start of loop over all cycles of calculation *********
C
  220 WRITE(6,3560)
      READ(5,2020,ERR=220) ncycl
      if (ncycl.LT.-100) then
C       *** start from the beginning ***
        GO TO 210
      elseif (ncycl.lt.0) then
C       *** input different lambda/alpha ***
        CALL HISTRY(itcycl,delta,alpha,lambda,nhist,
     $                  mhist,FHIST,IHIST,conold)
        goto 220
      elseif (ncycl.eq.0) then
C       *** stop calculations ***
        print *,
     $    'Do you really want to stop the calculation ? (0=yes)'
        iii=0
        read(5,2020) iii
        if (iii.ne.0) goto 220
C       *** stop and save density in file: ***
        goto 260 
      elseif (ncycl.gt. 0) then
C       *** normal case: positive number of cycles given ***
	icycl=1
        ibadruns=0
        igoodcyc=0
      endif
C
C     ***********************************************
C     ***** start of loop over NCYCL cycles *********
C     ***********************************************
C
240   continue
      if (ibadruns.gt.13) then
C        *** exit loop: save and display last sensible results ***
	 print *,'-------------------------------------------'
	 print *,'Exit with saving of data: divergence error!'
	 print *,'-------------------------------------------'
	 itcycl=igoodcyc
	 connew=conold
	 conf=confold
         cong=congold
	 delta=deltaold
	 rfact=rfacold
	 wrfact=wrfacold
C        *** take old density as result ***
	 do 245,ia=1,nasym
	    RDENp(ia)=RDEOp(ia)
	    RDENm(ia)=RDEOm(ia)
245      continue
         CALL CSTRCTF(masym,nasym,RDENp,RDENm
     $     ,RMIC
     $     ,RMIS
     $     ,RASYM,mref,nref,FCAL,sumBp,sumBm)
	 if (ngsw.eq.0) CALL CSTRCTG(mref,nrefF,nrefG,FCAL,GCAL,
     $                     PROF,NGRP,PMUL,PMULS)
	 goto 260
      endif
C ************************
      CALL CMAXENT(ngsw,mref,nrefF,nrefG,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,tnref,lambda
     $  ,RMIC
     $  ,RMIS
     $  ,PMUL,PMULS,RDEOp,RDEOm,RDENp,RDENm,
     $   RHELP1,RHELP2,RHELP3,denmin,sumBp,sumBm,rtBp,rtBm)
C
C ************************
C
      CALL NORMAL(masym,nasym,RDENp,RDENm,RASYM,rtasym)
      CALL CSTRCTF(masym,nasym,RDENp,RDENm
     $ ,RMIC
     $ ,RMIS
     $ ,RASYM,mref,nref,FCAL,sumBp,sumBm)
      if (ngsw.le.0) CALL CSTRCTG( mref,nrefF,nrefG,FCAL,GCAL,
     $                            PROF,NGRP,PMUL,PMULS)
      CALL CCALCON(ngsw,conf,cong,wrfact,rfact,connew,tnref,
     $  mref,nrefF,nrefG,FCAL,PROF,FOBS,SIGF,GCAL,GOBS,SIGG )
      delta=connew-conold
C
      WRITE(6,3060) itcycl
      WRITE(6,3030) connew,conf,cong,delta,rfact,wrfact
      IF (delta.GE.0.0 .or. 
     +    DABS(delta/conold).gt.0.1 ) then
C        *** divergence error: choose new lambda and restart calc. ***
C        *** or: at start of calculation too large step in contstraint
	 ibadruns=ibadruns+1
	 lambda=lambda/2
         nhist=nhist+1
	 if (nhist.gt.mhist) nhist=mhist
         FHIST(1,nhist)=lambda*2
         FHIST(2,nhist)=lambda
         FHIST(3,nhist)=conold
         IHIST(nhist)=itcycl
         write(*,'(a,i3,a,f13.8)') 'Bad cycles number:',ibadruns,
     $    '   New lambda=',lambda
C        *** call back F_cal, G_cal of last successful cycle ***
C        *** nref=nrefF + (No. of reflections in groups) ***
         DO 242 iref=1,nref
  	   FCAL(1,iref)=FCALO(1,iref)
	   FCAL(2,iref)=FCALO(2,iref)
242      CONTINUE
         IF (NGSW.le.0) THEN
           DO 243 iref=1,nrefG
             GCAL(iref)=GCALO(iref)
	     GCAL(iref)=GCALO(iref)
243        CONTINUE
         ENDIF
C        *** restart calculation with changed lambda ***
         GO TO 240
      ENDIF 
C     *** this cyle was successful: delta < 0 ***
C     *** new density rden --> rdeo ***
      CALL RENEW(masym,nasym,RDEOp,RDEOm,RDENp,RDENm)
C     *** increase lambda a bit ***
      lambda=lambda*1.02
C     *** memorize a lot of this cycle ***
      ibadruns=0
      igoodcyc=itcycl
      confold=conf
      congold=cong
      deltaold=delta
      rfacold=rfact
      wrfacold=wrfact
      DO 252 iref=1,nref
         FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
252   CONTINUE
      IF (ngsw.le.0) THEN
         DO 253 iref=1,nrefG
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
253      CONTINUE
      ENDIF
C     *** increase cycle counters ***
      itcycl=itcycl+1
      icycl=icycl+1
C     *** SAVE DENSITY DATA after every xxth cycle ***
      if (mod(itcycl-1,mautosave).eq.0) then
         print *,'Automatically saving density data ...'
	 CALL SAVEDEN(filden,
     $      masym,nasym,title,IASYM,RDENp,RDENm,CELL,NPIX
     $      ,nsym,ncen,nsub,IROT,RTRS,RSUB,rtBp,rtBm )
      endif
C
      IF (connew.LE.ALPHA) then
C        *** calculation finished ***  
	 GO TO 260
      endif
      conold=connew
      IF (ngsw.EQ.-1) ngsw=0
      IF (icycl.LE.ncycl) GO TO 240
C
C     ***** end of loop over NCYCL cycles *********
C
C     CALL DSPTIM(6,itcycl)
      GO TO 220
C
C     ==========================================================
C     ***** end of loop over all cycles of calculation *********
C     ==========================================================
C
C     **********  DISPLAY & OUTPUT ANALYSIS RESULT.  ***********
C
260   write(6,3000)
      itcycl=itcycl-1
C
C*** SAVE DENSITY DATA. ***
C
      CALL SAVEDEN(filden,
     $  masym,nasym,title,IASYM,RDENp,RDENm,CELL,NPIX
     $  ,nsym,ncen,nsub,IROT,RTRS,RSUB,rtBp,rtBm )
C
C*** CALCULATE ENTROPY-FUNCTIONS
      CALL CALFN(masym,nasym,entj,entsp,entsm,rtasym,
     $   RDENp,RDENm,RDEOp,RDEOm,RASYM,
     $   qfunc,lambda,connew)
C
C*** DISPLAY INTO STANDARD OUTPUT ***
      CALL DSPRESUL(6,
     $  connew,conf,cong,delta,rfact,wrfact,entsp,entsm,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,itcycl,
     $  mref,nrefF,nrefG,FOBS,FCAL,SIGF,PROF,
     $  NGRP,GOBS,GCAL,SIGG,IDX)
C
C*** DISPLAY INTO DOCUMENT FILE ***
      CALL DSPRESUL(30,
     $  connew,conf,cong,delta,rfact,wrfact,entsp,entsm,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,itcycl,
     $  mref,nrefF,nrefG,FOBS,FCAL,SIGF,PROF,
     $  NGRP,GOBS,GCAL,SIGG,IDX)
C
C*** DISPLAY FIRST DENSITY VALUES
      write(*,*) 'First 10 values of density:'
      write(30,*) 'First 10 values of density:'
      do i=1,10
        write(*,1090) 
     $  (iasym(j,i),j=1,3),rtBp*RDENp(i),rtBm*RDENm(i)
        write(30,1090) 
     $  (iasym(j,i),j=1,3),rtBp*RDENp(i),rtBm*RDENm(i)
      end do
      CLOSE(30)
      print *
C
C*** END OF PROGRAM ***
C
      STOP
      END
